@@ -1,9 +1,9 @@
-static char rcsid[] = "$Id: PctestIpv4.cc,v 1.21 2000/03/22 00:14:34 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv4.cc,v 1.28 2001/01/22 22:02:16 bmah Exp $";
 //
-// $Id: PctestIpv4.cc,v 1.21 2000/03/22 00:14:34 bmah Exp $
+// $Id: PctestIpv4.cc,v 1.28 2001/01/22 22:02:16 bmah Exp $
 //
 // PctestIpv4.cc
-// Bruce A. Mah <bmah@ca.sandia.gov>
+// Bruce A. Mah <bmah@acm.org>
 //
 // This work was first produced by an employee of Sandia National
 // Laboratories under a contract with the U.S. Department of Energy.
@@ -18,6 +18,7 @@ static char rcsid[] = "$Id: PctestIpv4.cc,v 1.21 2000/03/22 00:14:34 bmah Exp $"
 //
 
 #include <sys/types.h>
+#include <sys/param.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
@@ -29,6 +30,120 @@ static char rcsid[] = "$Id: PctestIpv4.cc,v 1.21 2000/03/22 00:14:34 bmah Exp $"
 #include "pc.h"
 #include "PctestIpv4.h"
 
+//
+// PctestIpv4::SetOriginName
+//
+// Input:  origin hostname
+//
+// Output:  success code (negative if error)
+//
+// Attempt to set the origin of probe packets emanating from this host,
+// for tests where it is applicable, using protocol family-dependent
+// resolution as necessary.  If no origin hostname is given, 
+// then use the target address to compute the correct outgoing interface.
+// originName and originAddress are set.
+//
+int PctestIpv4::SetOriginName(char *o)
+{
+
+    // If nothing was passed in, then bind a dummy socket to try
+    // to figure out where packets exit this host.
+    if (o == NULL) {
+
+	int dummySock;
+	struct sockaddr_in dummyAddr, localAddr;
+#ifdef HAVE_SOCKLEN_T
+	socklen_t localAddrLength;
+#else /* HAVE__SOCKLEN_T */
+	size_t localAddrLength;
+#endif /* HAVE_SOCKLEN_T */
+
+	// Create socket, then connect to it, and then read out
+	// the local socket address with a getsockname call.
+	dummySock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+	if (dummySock < 0) {
+	    perror("socket");
+	    return -1;
+	}
+
+	memset((void *) &dummyAddr, 0, sizeof(struct sockaddr_in));
+#ifdef HAVE_SOCKADDR_SA_LEN
+	dummyAddr.sin_len = sizeof(struct sockaddr_in);
+#endif /* HAVE_SOCKADDR_SA_LEN */
+	dummyAddr.sin_family = AF_INET;
+	dummyAddr.sin_port = htons(32768); // port number irrelevant
+	memcpy(&dummyAddr.sin_addr, &targetAddress, 
+	       sizeof(struct in_addr));
+
+	if (connect(dummySock, (sockaddr *) &dummyAddr, 
+		    sizeof(struct sockaddr_in)) < 0) {
+	    perror("connect");
+	    return -1;
+	}
+
+	localAddrLength = sizeof(sockaddr_in);
+	memset((void *) &localAddr, 0, sizeof(struct sockaddr_in));
+	if (getsockname(dummySock, (struct sockaddr *) &localAddr,
+			&localAddrLength) < 0) {
+	    perror("getsockname");
+	    return -1;
+	}
+
+	// Got the local address, now do a reverse DNS looup
+	memcpy(&originAddress, &localAddr.sin_addr, sizeof(struct in_addr));
+	originName = strdup(GetName((char *) &originAddress));
+	if (originName == NULL) {
+	    fprintf(stderr, "Couldn't allocate memory for origin hostname.\n");
+	    return -1;
+	}
+
+	close(dummySock);
+    }
+
+    // User gave a source name/address.  Attempt to resolve, if possible.
+    else {
+
+	struct hostent *host;	// resolver host entry
+
+	host = gethostbyname(o);
+
+	// Resolver failed
+	if (host == NULL) {
+
+#ifdef HAVE_HERROR
+	    herror(o);
+#else
+	    fprintf(stderr, "%s: Host not found\n", o);
+#endif /* HAVE_HERROR */
+
+	    memset((void *) &originAddress, 0, sizeof(struct in_addr));
+	    originName = strdup(o);
+
+	    if (originName == NULL) {
+		fprintf(stderr, "Couldn't allocate space for origin hostname.\n");
+		return -1;
+	    }
+	    return -1;
+	}
+
+	IF_DEBUG(3, fprintf(stderr, "h_name = %s\n", host->h_name));
+	IF_DEBUG(3, fprintf(stderr, "h_length = %d\n", host->h_length));
+	IF_DEBUG(3, fprintf(stderr, "h_addr_list[0] = %x\n", *((int *)(host->h_addr_list[0]))));
+	     
+	// Get IP address
+	memcpy(&originAddress, host->h_addr_list[0], host->h_length);
+
+	// Make a copy of the canonical hostname
+	originName = strdup(host->h_name);
+	if (originName == NULL) {
+	    fprintf(stderr, "Couldn't allocate memory for origin hostname.\n");
+	    return -1;
+	}
+    }
+
+    return 0;
+}
+
 //
 // PctestIpv4::SetTargetName
 //
@@ -83,7 +198,7 @@ int PctestIpv4::SetTargetName(char *t)
     targetSocketAddress.sin_len = sizeof(struct sockaddr_in);
 #endif /* HAVE_SOCKADDR_SA_LEN */
     targetSocketAddress.sin_family = AF_INET;
-    targetSocketAddress.sin_port = 0; // set on each test
+    targetSocketAddress.sin_port = htons(0); // set on each test
     memcpy(&targetSocketAddress.sin_addr, host->h_addr_list[0], host->h_length);
 
     // Make a copy of the canonical hostname
@@ -98,7 +213,7 @@ int PctestIpv4::SetTargetName(char *t)
     // Now get the two sockets we need, one to send UDP datagrams
     // and another to receive ICMP packets.  The UDP socket is
     // easy, but we need to use a raw socket for ICMP.
-    socketOut = GetSocketOut();
+/*    socketOut = GetSocketOut();
     if (socketOut < 0) {
 	return socketOut;
     }
@@ -107,7 +222,7 @@ int PctestIpv4::SetTargetName(char *t)
     if (socketIn < 0) {
 	return socketIn;
     }
-
+*/
     return 0;
 
 }
@@ -189,43 +304,3 @@ char *PctestIpv4::GetName(void *a)
 
 }
 
-//
-// PctestIpv4::GetMinSize
-//
-// Input:  None
-//
-// Output:  Minimum packet size possible for this protocol (in return
-// value).
-//
-int PctestIpv4::GetMinSize() 
-{
-    return (sizeof(ip) + sizeof(udphdr) + 4);
-}
-
-//
-// PctestIpv4::GetAction
-//
-// Input:  a test record
-//
-// Output:  action code
-//
-// Figure out the meaning of a particular combination of ICMPv4 type
-// and code values.
-//
-PctestActionType PctestIpv4::GetAction(TestRecord &tr) 
-{
-    if (tr.icmpType == ICMP_TIMXCEED) {
-	return PctestActionValid;
-    }
-    else if ((tr.icmpType == ICMP_UNREACH) &&
-	     (tr.icmpCode == ICMP_UNREACH_PORT)) {
-	return PctestActionValidLasthop;
-    }
-    else if ((tr.icmpType == ICMP_UNREACH) &&
-	     (tr.icmpCode == ICMP_UNREACH_FILTER_PROHIB)) {
-	return PctestActionFiltered;
-    }
-    else {
-	return PctestActionAbort;
-    }
-}
