@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv4Raw.cc,v 1.16 2001/01/19 22:18:49 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv4Raw.cc,v 1.17 2001/01/26 17:21:12 bmah Exp $";
 //
-// $Id: PctestIpv4Raw.cc,v 1.16 2001/01/19 22:18:49 bmah Exp $
+// $Id: PctestIpv4Raw.cc,v 1.17 2001/01/26 17:21:12 bmah Exp $
 //
 // PctestIpv4Raw.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -231,7 +231,7 @@ int PctestIpv4Raw::Test(TestRecord &tr)
 	    IF_DEBUG(2, fprintf(stderr, "Response packet received\n"));
 
 	    const int icmpPacketSize = 1024 + tr.size;
-	    char icmpPacket[icmpPacketSize];
+	    char icmpPacket[IP_MAXPACKET];
 	
 	    rc = read(socketIn, icmpPacket, icmpPacketSize);
 	    if (rc < 0) {
