@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv4Udp.cc,v 1.16 2001/01/19 22:18:51 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv4Udp.cc,v 1.17 2001/01/26 17:21:14 bmah Exp $";
 //
-// $Id: PctestIpv4Udp.cc,v 1.16 2001/01/19 22:18:51 bmah Exp $
+// $Id: PctestIpv4Udp.cc,v 1.17 2001/01/26 17:21:14 bmah Exp $
 //
 // PctestIpv4Udp.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -180,8 +180,8 @@ int PctestIpv4Udp::Test(TestRecord &tr)
 	    IF_DEBUG(2, fprintf(stderr, "Response packet received\n"));
 
 	    const int icmpPacketSize = 1024 + tr.size;
-	    char icmpPacket[icmpPacketSize];
-	
+	    char icmpPacket[IP_MAXPACKET];
+	    
 	    rc = read(socketIn, icmpPacket, icmpPacketSize);
 	    if (rc < 0) {
 		perror("read");
