@@ -1,9 +1,9 @@
-static char rcsid[] = "$Id: PctestIpv6.cc,v 1.12 2000/05/01 20:25:41 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv6.cc,v 1.23 2001/01/22 22:02:17 bmah Exp $";
 //
-// $Id: PctestIpv6.cc,v 1.12 2000/05/01 20:25:41 bmah Exp $
+// $Id: PctestIpv6.cc,v 1.23 2001/01/22 22:02:17 bmah Exp $
 //
 // PctestIpv6.cc
-// Bruce A. Mah <bmah@ca.sandia.gov>
+// Bruce A. Mah <bmah@acm.org>
 //
 // This work was first produced by an employee of Sandia National
 // Laboratories under a contract with the U.S. Department of Energy.
@@ -18,6 +18,7 @@ static char rcsid[] = "$Id: PctestIpv6.cc,v 1.12 2000/05/01 20:25:41 bmah Exp $"
 //
 
 #include <sys/types.h>
+#include <sys/param.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
@@ -29,6 +30,126 @@ static char rcsid[] = "$Id: PctestIpv6.cc,v 1.12 2000/05/01 20:25:41 bmah Exp $"
 #include "pc.h"
 #include "PctestIpv6.h"
 
+//
+// PctestIpv4::SetOriginName
+//
+// Input:  origin hostname
+//
+// Output:  success code (negative if error)
+//
+// Attempt to set the origin of probe packets emanating from this host,
+// for tests where it is applicable, using protocol family-dependent
+// resolution as necessary.  If no origin hostname is given, 
+// then use the target address to compute the correct outgoing interface.
+// originName and originAddress are set.
+//
+int PctestIpv6::SetOriginName(char *o)
+{
+
+    // If nothing was passed in, then bind a dummy socket to try to 
+    // determine the output interface.
+    if (o == NULL) {
+	int dummySock;
+	struct sockaddr_in6 dummyAddr, localAddr;
+#ifdef HAVE_SOCKLEN_T
+	socklen_t localAddrLength;
+#else /* HAVE_SOCKLEN_T */
+	size_t localAddrLength;
+#endif /* HAVE_SOCKLEN_T */
+
+	// Create socket, then connect to it, and then read out
+	// the local socket address with a getsockname call.
+	dummySock = socket(AF_INET6, SOCK_DGRAM, IPPROTO_UDP);
+	if (dummySock < 0) {
+	    perror("socket");
+	    return -1;
+	}
+
+	memset((void *) &dummyAddr, 0, sizeof(struct sockaddr_in6));
+#ifdef HAVE_SOCKADDR_SA_LEN
+	dummyAddr.sin6_len = sizeof(struct sockaddr_in6);
+#endif /* HAVE_SOCKADDR_SA_LEN */
+	dummyAddr.sin6_family = AF_INET6;
+	dummyAddr.sin6_port = htons(32768); // port number irrelevant
+	memcpy(&dummyAddr.sin6_addr, &targetAddress, 
+	       sizeof(struct in6_addr));
+
+	if (connect(dummySock, (sockaddr *) &dummyAddr, 
+		    sizeof(struct sockaddr_in6)) < 0) {
+	    perror("connect");
+	    return -1;
+	}
+	
+	localAddrLength = sizeof(sockaddr_in6);
+	memset((void *) &localAddr, 0, sizeof(struct sockaddr_in6));
+	if (getsockname(dummySock, (struct sockaddr *) &localAddr,
+			&localAddrLength) < 0) {
+	    perror("getsockname");
+	    return -1;
+	}
+
+	// Got the local address, now do a reverse DNS looup
+	memcpy(&originAddress, &localAddr.sin6_addr, sizeof(struct in6_addr));
+	originName = strdup(GetName((char *) &originAddress));
+	if (originName == NULL) {
+	    fprintf(stderr, "Couldn't allocate memory for origin hostname.\n");
+	    return -1;
+	}
+
+	close(dummySock);
+
+    }
+
+    else {
+	// Attempt to resolve, if possible.  Instead of using AI_DEFAULT,
+	// we want to force the use of IPv6 addresses (no mapped or IPv4
+	// addresses, since the test code relies on knowing exactly what
+	// protocol is being used on the wire).
+	struct addrinfo *host = NULL;
+	struct addrinfo hints;
+	int ecode;
+	int error_num;		// resolver error code
+
+	memset(&hints, 0, sizeof(hints));
+	hints.ai_family = PF_INET6;
+	hints.ai_flags = AI_CANONNAME;
+	ecode = getaddrinfo(o, NULL, &hints, &host);
+
+	// Resolver failed
+	if (host == NULL) {
+	    fprintf(stderr, "%s: %s\n", o, gai_strerror(ecode));
+
+	    memset((void *) &originAddress, 0, sizeof(struct in6_addr));
+	    originName = strdup(o);
+
+	    if (originName == NULL) {
+		fprintf(stderr, "Couldn't allocate space for origin hostname.\n");
+		return -1;
+	    }
+	    return -1;
+	}
+
+	IF_DEBUG(3, fprintf(stderr, "h_name = %s\n", host->ai_canonname));
+	IF_DEBUG(3, fprintf(stderr, "h_length = %d\n", host->ai_addrlen));
+
+	// Get IP address
+	memcpy(&originAddress,
+	       &((struct sockaddr_in6 *) host->ai_addr)->sin6_addr,
+	       sizeof(originAddress));
+
+	// Make a copy of the canonical hostname
+	originName = strdup(host->ai_canonname);
+	if (originName == NULL) {
+	    fprintf(stderr, "Couldn't allocate memory for hostname.\n");
+	    return -1;
+	}
+
+	freeaddrinfo(host);
+    }
+
+    return 0;
+}
+
 //
 // PctestIpv6::SetTargetName
 //
@@ -57,6 +178,7 @@ int PctestIpv6::SetTargetName(char *t)
     // protocol is being used on the wire).
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = PF_INET6;
+    hints.ai_flags = AI_CANONNAME;
     ecode = getaddrinfo(t, NULL, &hints, &host);
 
     // Resolver failed
@@ -83,7 +205,7 @@ int PctestIpv6::SetTargetName(char *t)
     // Note:  Only BSD4.3Reno and later have sin_len in struct
     // sockaddr_in, so we need to test for it.
     //
-    // XXX Really what we ought to have is a check in autoconf for
+    // Really what we ought to have is a check in autoconf for
     // sockaddr_in6.sin6_len, rather than relying on the (already
     // existing) check for sockaddr.sin_len.  There are two instances
     // in this file.  (On the other hand, any system that has
@@ -100,15 +222,14 @@ int PctestIpv6::SetTargetName(char *t)
     // have 24-bit flow labels).  If we want to support setting a
     // flow label value, we can OR it with the value below.
     targetSocketAddress.sin6_flowinfo = htonl(Tos << 20); 
-    targetSocketAddress.sin6_port = 0; // set on each test
+    targetSocketAddress.sin6_port = htons(0); // set on each test
 
-    /* XXX We could just copy host->ai_addr instead ? */
     memcpy(&targetSocketAddress.sin6_addr,
 	   &((struct sockaddr_in6 *) host->ai_addr)->sin6_addr,
            sizeof(struct in6_addr));
 
     // Make a copy of the canonical hostname
-    len = strlen(t);
+    len = strlen(host->ai_canonname);
 
     targetName = (char *) calloc(len + 1, sizeof(char));
     if (targetName == NULL) {
@@ -116,22 +237,9 @@ int PctestIpv6::SetTargetName(char *t)
 	return -1;
     }
 
-    strncpy(targetName, t, len + 1);
+    strncpy(targetName, host->ai_canonname, len + 1);
     freeaddrinfo(host);
 
-    // Now get the two sockets we need, one to send UDP datagrams
-    // and another to receive ICMP packets.  The UDP socket is
-    // easy, but we need to use a raw socket for ICMP.
-    socketOut = GetSocketOut();
-    if (socketOut < 0) {
-	return socketOut;
-    }
-
-    socketIn = GetSocketIn();
-    if (socketIn < 0) {
-	return socketIn;
-    }
-
     return 0;
 
 }
@@ -165,12 +273,11 @@ int PctestIpv6::GetSocketIn() {
 	return socketIn;
     }
 
-    // ICMPv6 message filtering.  We only want destination unreachable
-    // and time exceeded (for now).  We might want the ability to
-    // process other types in the future
+    // ICMPv6 message filtering.
     ICMP6_FILTER_SETBLOCKALL(&filt);
     ICMP6_FILTER_SETPASS(ICMP6_DST_UNREACH, &filt);
     ICMP6_FILTER_SETPASS(ICMP6_TIME_EXCEEDED, &filt);
+    ICMP6_FILTER_SETPASS(ICMP6_ECHO_REPLY, &filt);
     rc = setsockopt(socketIn, IPPROTO_ICMPV6, ICMP6_FILTER, &filt, sizeof(filt));
     if (rc < 0) {
 	perror("setsockopt(ICMP_FILTER)");
@@ -257,43 +364,4 @@ char *PctestIpv6::GetName(void *a)
 
 }
 
-//
-// PctestIpv6::GetMinSize
-//
-// Input:  None
-//
-// Output:  Minimum packet size possible for this protocol (in return
-// value).
-//
-int PctestIpv6::GetMinSize() 
-{
-    return (sizeof(ip6_hdr) + sizeof(udphdr) + 6);
-}
 
-//
-// PctestIpv6::GetAction
-//
-// Input:  a test record
-//
-// Output:  action code
-//
-// Figure out the meaning of a particular combination of ICMPv4 type
-// and code values.
-//
-PctestActionType PctestIpv6::GetAction(TestRecord &tr) 
-{
-    if (tr.icmpType == ICMP6_TIME_EXCEEDED) {
-	return PctestActionValid;
-    }
-    else if ((tr.icmpType == ICMP6_DST_UNREACH) &&
-	     (tr.icmpCode == ICMP6_DST_UNREACH_NOPORT)) {
-	return PctestActionValidLasthop;
-    }
-    else if ((tr.icmpType == ICMP6_DST_UNREACH) &&
-	     (tr.icmpCode == ICMP6_DST_UNREACH_ADMIN)) {
-	return PctestActionFiltered;
-    }
-    else {
-	return PctestActionAbort;
-    }
-}
