@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv6Udp.cc,v 1.24 2001/01/22 17:36:00 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv6Udp.cc,v 1.26 2001/01/27 00:28:45 bmah Exp $";
 //
-// $Id: PctestIpv6Udp.cc,v 1.24 2001/01/22 17:36:00 bmah Exp $
+// $Id: PctestIpv6Udp.cc,v 1.26 2001/01/27 00:28:45 bmah Exp $
 //
 // PctestIpv6Udp.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -185,7 +185,7 @@ int PctestIpv6Udp::Test(TestRecord &tr)
 	    IF_DEBUG(2, fprintf(stderr, "Response packet received\n"));
 
 	    const int icmpPacketSize = 1500 + tr.size;
-	    char icmpPacket[icmpPacketSize];
+	    char icmpPacket[IPV6_MAXPACKET];
 	    struct msghdr msg;	// msghdr is for recvmsg
 	    struct iovec iov[1];
 	    int controlsize;
@@ -207,7 +207,7 @@ int PctestIpv6Udp::Test(TestRecord &tr)
 	    // draft-ietf-ipngwg-2292bis becomes an RFC.  Until
 	    // then, he has a small hack to fix this, slightly
 	    // modified by bmah.
-#ifdef __sun__
+#if (defined(__sun__) || (defined(__sun)))
 #ifndef CMSG_SPACE
 #define CMSG_SPACE(length) \
         (_CMSG_DATA_ALIGN(sizeof(struct cmsghdr)) + _CMSG_HDR_ALIGN(length))
