@@ -1,9 +1,9 @@
 // -*- c++ -*-
 //
-// $Id: PctestIpv6Udp.h,v 1.2 2000/01/19 17:14:42 bmah Exp $
+// $Id: PctestIpv6Udp.h,v 1.5 2001/01/02 18:46:15 bmah Exp $
 //
 // PctestIpv6Udp.h
-// Bruce A. Mah <bmah@ca.sandia.gov>
+// Bruce A. Mah <bmah@acm.org>
 //
 // This work was first produced by an employee of Sandia National
 // Laboratories under a contract with the U.S. Department of Energy.
@@ -52,6 +52,8 @@ class PctestIpv6Udp : public PctestIpv6 {
 
     virtual int GetSocketOut();
     virtual int Test(TestRecord &tr);
+    virtual int GetMinSize();
+    virtual PctestActionType GetAction(int icmpType, int icmpCode);
    
   protected:
 
