@@ -1,9 +1,9 @@
-static char rcsid[] = "$Id: TestRecord.cc,v 1.11 2000/02/03 18:59:14 bmah Exp $";
+static char rcsid[] = "$Id: TestRecord.cc,v 1.20 2001/01/18 22:12:45 bmah Exp $";
 //
-// $Id: TestRecord.cc,v 1.11 2000/02/03 18:59:14 bmah Exp $
+// $Id: TestRecord.cc,v 1.20 2001/01/18 22:12:45 bmah Exp $
 //
 // TestRecord.cc
-// Bruce A. Mah <bmah@ca.sandia.gov>
+// Bruce A. Mah <bmah@acm.org>
 //
 // This work was first produced by an employee of Sandia National
 // Laboratories under a contract with the U.S. Department of Energy.
@@ -20,9 +20,6 @@ static char rcsid[] = "$Id: TestRecord.cc,v 1.11 2000/02/03 18:59:14 bmah Exp $"
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#ifdef HAVE_IPV6
-#include <netinet/in.h>
-#endif /* HAVE_IPV6 */
 #include <arpa/inet.h>
 
 #include "pc.h"
@@ -56,7 +53,7 @@ char *TestRecord::htoa(Pctest *pct)
 #else
     sprintf(buffer2, 
 #endif /* HAVE_SNPRINTF */
-      "probe timestamp %ld.%06ld src %s ", tvstart.tv_sec, tvstart.tv_usec, pct->GetPrintableAddress(icmpDestAddress));
+      "probe t %ld.%06ld ", tvstart.tv_sec, tvstart.tv_usec);
     strncat(buffer, buffer2, buflen);
 
 #ifdef HAVE_SNPRINTF
@@ -64,15 +61,7 @@ char *TestRecord::htoa(Pctest *pct)
 #else
     sprintf(buffer2, 
 #endif /* HAVE_SNPRINTF */
-      "dest %s ", pct->GetPrintableAddress(targetAddress));
-    strncat(buffer, buffer2, buflen);
-
-#ifdef HAVE_SNPRINTF
-    snprintf(buffer2, buflen, 
-#else
-    sprintf(buffer2, 
-#endif /* HAVE_SNPRINTF */
-      "hops %d bytes %d icmpsrc %s icmptype %d icmpcode %d rtt %ld.%06ld", hops, size, pct->GetPrintableAddress(icmpSourceAddress), icmpType, icmpCode, tv.tv_sec, tv.tv_usec);
+      "h %d b %d addr %s res %d rtt %ld.%06ld rb %d", hops, size, pct->GetPrintableAddress(icmpSourceAddress), result, tv.tv_sec, tv.tv_usec, replsize);
     strncat(buffer, buffer2, buflen);
 
     return buffer;
@@ -81,25 +70,28 @@ char *TestRecord::htoa(Pctest *pct)
 //
 // TestRecord::atoh
 //
-// Input:
+// Input: input string, Pctest record
 //
-// Output:
+// Output: pointer to a new TestRecord, NULL if an error
 //
 // Parse the ASCII representation described above and make up a new
 // TestRecord with demarshalled parameters.  The caller "owns"
 // this object and is responsible for deallocating it.
 //
-TestRecord *TestRecord::atoh(char *s) 
+// The Pctest record is necessary to determine the address family
+// that needs to be used when parsing addresses on this line.
+//
+TestRecord *TestRecord::atoh(char *s, Pctest *pct) 
 {
 
     TestRecord *tr;
 
     char srcChars[256], destChars[256], icmpsrcChars[256];
     float tvstartFloat, tvFloat;
-    int hops, size;
-    int icmpType, icmpCode;
+    int hops, size, replsize;
+    int result;
 
-    if (sscanf(s, "probe timestamp %f src %s dest %s hops %d bytes %d icmpsrc %s icmptype %d icmpcode %d rtt %f", &tvstartFloat, srcChars, destChars, &hops, &size, icmpsrcChars, &icmpType, &icmpCode, &tvFloat) == 9) {
+    if (sscanf(s, "probe t %f h %d b %d addr %s res %d rtt %f rb %d", &tvstartFloat, &hops, &size, icmpsrcChars, &result, &tvFloat, &replsize) == 7) {
 
 	tr = new TestRecord;
 	tr->size = size;
@@ -109,20 +101,29 @@ TestRecord *TestRecord::atoh(char *s)
 	tr->tv.tv_sec = (long) tvFloat;
 	tr->tv.tv_usec = (long) ((tvFloat - ((long) tvFloat)) * 1000000.0);
 
-	tr->targetAddress = new char[sizeof(in_addr)];
-	((in_addr *) tr->targetAddress)->s_addr = inet_addr(destChars);
-	tr->targetAddressLength= sizeof(in_addr);
+	// Parse the gateway address in an address-family dependant
+	// way.
+	int af = pct->GetAddressFamily();
+	if (af == AF_INET) {
+	    tr->icmpSourceAddress = new char[sizeof(in_addr)];
+	    ((in_addr *) tr->icmpSourceAddress)->s_addr = 
+		inet_addr(icmpsrcChars);
+	    tr->icmpSourceAddressLength= sizeof(in_addr);
+	}
+#ifdef HAVE_IPV6
+	else if (af == AF_INET6) {
+	    tr->icmpSourceAddress = new char[sizeof(in6_addr)];
+	    inet_pton(AF_INET6, icmpsrcChars, (void *) tr->icmpSourceAddress);
+	    tr->icmpSourceAddressLength= sizeof(in6_addr);
+	}
+#endif // HAVE_IPV6
+	else {
+	    fprintf(stderr, "Unknown address family: %s\n", s);
+	    return NULL;
+	}
 
-	tr->icmpSourceAddress = new char[sizeof(in_addr)];
-	((in_addr *) tr->icmpSourceAddress)->s_addr = inet_addr(icmpsrcChars);
-	tr->icmpSourceAddressLength= sizeof(in_addr);
-
-	tr->icmpDestAddress = new char[sizeof(in_addr)];
-	((in_addr *) tr->icmpDestAddress)->s_addr = inet_addr(srcChars);
-	tr->icmpDestAddressLength= sizeof(in_addr);
-
-	tr->icmpType = icmpType;
-	tr->icmpCode = icmpCode;
+	tr->result = (PctestActionType) result;
+	tr->replsize = replsize;
 
 	return tr;
 
@@ -134,61 +135,4 @@ TestRecord *TestRecord::atoh(char *s)
 
 }
 
-#ifdef HAVE_IPV6
-//
-// TestRecord::atoh6
-//
-// Input:
-//
-// Output:
-//
-// Parse the ASCII representation described above and make up a new
-// TestRecord with demarshalled parameters.  The caller "owns"
-// this object and is responsible for deallocating it.
-//
-TestRecord *TestRecord::atoh6(char *s) 
-{
-
-    TestRecord *tr;
-
-    char srcChars[256], destChars[256], icmpsrcChars[256];
-    float tvstartFloat, tvFloat;
-    int hops, size;
-    int icmpType, icmpCode;
-
-    if (sscanf(s, "probe timestamp %f src %s dest %s hops %d bytes %d icmpsrc %s icmptype %d icmpcode %d rtt %f", &tvstartFloat, srcChars, destChars, &hops, &size, icmpsrcChars, &icmpType, &icmpCode, &tvFloat) == 9) {
-
-	tr = new TestRecord;
-	tr->size = size;
-	tr->hops = hops;
-	tr->tvstart.tv_sec = (long) tvstartFloat;
-	tr->tvstart.tv_usec = (long) ((tvstartFloat - ((long) tvstartFloat)) * 1000000.0);
-	tr->tv.tv_sec = (long) tvFloat;
-	tr->tv.tv_usec = (long) ((tvFloat - ((long) tvFloat)) * 1000000.0);
-
-	tr->targetAddress = new char[sizeof(in6_addr)];
-	inet_pton(AF_INET6, destChars, (void *) tr->targetAddress);
-	tr->targetAddressLength= sizeof(in6_addr);
-
-	tr->icmpSourceAddress = new char[sizeof(in6_addr)];
-	inet_pton(AF_INET6, icmpsrcChars, (void *) tr->icmpSourceAddress);
-	tr->icmpSourceAddressLength= sizeof(in6_addr);
-
-	tr->icmpDestAddress = new char[sizeof(in6_addr)];
-	inet_pton(AF_INET6, srcChars, (void *) tr->icmpDestAddress);
-	tr->icmpDestAddressLength= sizeof(in6_addr);
-
-	tr->icmpType = icmpType;
-	tr->icmpCode = icmpCode;
-
-	return tr;
-
-    }
-    else {
-	fprintf(stderr, "Syntax error: %s\n");
-	return NULL;
-    }
-
-}
-#endif /* HAVE_IPV6 */
 
