@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: main.cc,v 1.82 2001/01/22 17:31:59 bmah Exp $";
+static char rcsid[] = "$Id: main.cc,v 1.83 2001/01/27 00:27:08 bmah Exp $";
 //
-// $Id: main.cc,v 1.82 2001/01/22 17:31:59 bmah Exp $
+// $Id: main.cc,v 1.83 2001/01/27 00:27:08 bmah Exp $
 //
 // main.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -1601,7 +1601,7 @@ endreps:
 #ifdef HAVE_SNMP
 	if (!ReadFilename && SnmpFlag) {
 	    GetIfInfo *gifp;
-	    gifp = new GetIfInfo(checkAddress, pct);
+	    gifp = new GetIfInfo(tr->icmpSourceAddress, pct);
 	    PrintIfInfo(gifp);
 	    delete gifp;
 	}
